module Main (main) where

import           Control.Monad
import           Data.Foldable
import           Data.Version          (showVersion)
import           Language.C.Dependency
import           Options.Applicative
import           Paths_cdeps           (version)

includes' :: Parser [FilePath]
includes' = many $ strOption $ mconcat
    [ long "include"
    , short 'I'
    , help "Directories for inclusions."
    ]

noRecurse :: Parser Bool
noRecurse = switch $ mconcat
    [ long "no-recurse"
    , short 'r'
    , help "Do not search for dependencies recursively."
    ]

data Command = Dump FilePath [FilePath] Bool

dump :: Parser Command
dump = Dump
    <$> target
    <*> includes'
    <*> noRecurse

cCompletions :: Mod ArgumentFields a
cCompletions = completer . bashCompleter $ "file -X '!*.c' -o plusdirs"

target :: Parser FilePath
target = argument str
    (metavar "SOURCE"
    <> help "C source file"
    <> cCompletions)

go :: ([FilePath] -> FilePath -> IO [FilePath]) -> [FilePath] -> FilePath -> IO ()
go = ((traverse_ putStrLn <=<) .)

run :: Command -> IO ()
run (Dump cSrc is False) = go getAll is cSrc
run (Dump cSrc is True)  = go getCDepends is cSrc

versionInfo :: Parser (a -> a)
versionInfo = infoOption ("cdeps version: " ++ showVersion version) (short 'V' <> long "version" <> help "Show version")

wrapper :: ParserInfo Command
wrapper = info (helper <*> versionInfo <*> dump)
    (fullDesc
    <> progDesc "CDeps finds local dependencies for builds"
    <> header "cdeps - a tool for tracking build dependencies")

main :: IO ()
main = execParser wrapper >>= run
